@@ -7,12 +7,14 @@ import urllib
 from mimetypes import types_map
 from datetime import datetime
 import json
+import string
 
 from django.utils import timezone
 from django.conf import settings
 from django.contrib.gis.gdal.error import OGRException
 from django.contrib.gis.geos import GEOSException, fromstr
 from django.http import HttpResponse
+from django.core.urlresolvers import resolve
 
 import bs4
 import requests
@@ -141,12 +143,19 @@ def download_to_stream(url, stream, silent=False, headers=None):
         source = None
         logger.info("Request to: %s" % url)
         source = requests.get(url, headers=headers)
-        assert source.status_code == 200, 'Request failed (status=%s)' % source.status_code
-        assert len(source.content) > 0, 'Request returned empty content'
+
+        status_error = 'Request on %s failed (status=%s)' % (url, source.status_code)
+        assert source.status_code == 200, status_error
+
+        content_error = 'Request on %s returned empty content' % url
+        assert len(source.content) > 0, content_error
+
     except (AssertionError, requests.exceptions.RequestException) as e:
         logger.exception(e)
+        logger.info('Headers sent: %s' % headers)
         if hasattr(source, 'content'):
-            logger.error(source.content[:150])
+            logger.info('Response: %s' % source.content[:150])
+
         if not silent:
             raise
 
@@ -162,7 +171,7 @@ def download_to_stream(url, stream, silent=False, headers=None):
             raise
 
     if isinstance(stream, HttpResponse):
-        stream.status = source.status_code
+        stream.status_code = source.status_code
         # Copy headers
         for header, value in source.headers.items():
             stream[header] = value
@@ -187,7 +196,7 @@ def convertit_url(url, from_type=None, to_type=None, proxy=False):
     return url
 
 
-def convertit_download(url, destination, from_type=None, to_type='application/pdf'):
+def convertit_download(url, destination, from_type=None, to_type='application/pdf', headers=None):
     # Mock for tests
     if getattr(settings, 'TEST', False):
         open(destination, 'wb').write("Mock\n")
@@ -195,7 +204,7 @@ def convertit_download(url, destination, from_type=None, to_type='application/pd
 
     url = convertit_url(url, from_type, to_type)
     fd = open(destination, 'wb') if isinstance(destination, basestring) else destination
-    download_to_stream(url, fd)
+    download_to_stream(url, fd, headers=headers)
 
 
 def capture_url(url, width=None, height=None, selector=None, waitfor=None):
@@ -247,7 +256,7 @@ def capture_map_image(url, destination, size=None, aspect=1.0):
                       waitfor='.leaflet-tile-loaded')
 
 
-def extract_attributes_html(url):
+def extract_attributes_html(url, request):
     """
     The tidy XHTML version of objects attributes.
 
@@ -256,11 +265,11 @@ def extract_attributes_html(url):
     With this, we save a lot of efforts, since we do have to build specific Appy.pod
     templates for each model.
     """
-    r = requests.get(url)
-    if r.status_code != 200:
-        raise ValueError('Could not reach %s' % url)
+    func, args, kwargs = resolve(url)
+    response = func(request, *args, **kwargs)
+    response.render()
 
-    soup = bs4.BeautifulSoup(r.content)
+    soup = bs4.BeautifulSoup(response.content)
     details = soup.find(id="properties")
     if details is None:
         raise ValueError('Content is of detail page is invalid')
@@ -290,3 +299,17 @@ def user_has_perm(user, perm):
     if user.is_anonymous():
         return perm in app_settings['ANONYMOUS_VIEWS_PERMS']
     return False
+
+
+def alphabet_enumeration(length):
+    """
+    Return list of letters : A, B, ... Z, AA, AB, ...
+    See mapentity/leaflet.enumeration.js
+    """
+    enums = []
+    alphabet = string.lowercase.upper()
+    for i in range(length):
+        enum = '' if i < 26 else alphabet[(i // 26) - 1]
+        enum += alphabet[i % 26]
+        enums.append(enum)
+    return enums
