@@ -1,11 +1,16 @@
+import logging
+
 from django.conf import settings
 from django.utils.importlib import import_module
 from django.core.management.base import BaseCommand
 
-from mapentity import registry, logger
+from mapentity import registry
 from mapentity.registry import create_mapentity_model_permissions
 
 
+logger = logging.getLogger(__name__)
+
+
 class Command(BaseCommand):
     help = "Create MapEntity models permissions"
 
