@@ -3,34 +3,32 @@ import logging
 from datetime import datetime
 
 from django.conf import settings
-from django.http import (HttpResponse, HttpResponseBadRequest,
-                         HttpResponseServerError)
+from django.http import HttpResponse, HttpResponseBadRequest
 from django.utils.translation import ugettext_lazy as _
 from django.utils.decorators import method_decorator
 from django.utils.encoding import force_text
+from django.views import static
 from django.views.generic.detail import DetailView
 from django.views.generic import View
 from django.views.generic.edit import CreateView, UpdateView, DeleteView
 from django.views.generic.list import ListView
-from django.contrib.gis.db.models import GeometryField
-from django.core.cache import get_cache
 from django.template.base import TemplateDoesNotExist
 from django.template.defaultfilters import slugify
 from django.contrib import messages
 from django.contrib.auth.decorators import login_required
-from djgeojson.views import GeoJSONLayerView
+from django.core.exceptions import PermissionDenied
 from djappypod.odt import get_template
 from djappypod.response import OdtTemplateResponse
 
-from ..settings import app_settings, API_SRID
+from ..settings import app_settings
 from .. import models as mapentity_models
 from ..helpers import convertit_url, download_to_stream, user_has_perm
-from ..decorators import save_history, view_permission_required, view_cache_latest
+from ..decorators import save_history, view_permission_required
+from ..forms import AttachmentForm
 from ..models import LogEntry, ADDITION, CHANGE, DELETION
-from ..serializers import GPXSerializer, CSVSerializer, DatatablesSerializer, ZipShapeSerializer
-from ..filters import MapEntityFilterSet
-from .base import history_delete
-from .mixins import ModelViewMixin, JSONResponseMixin, FormViewMixin
+from .. import serializers as mapentity_serializers
+from .base import history_delete, BaseListView
+from .mixins import (ModelViewMixin, FormViewMixin)
 
 
 logger = logging.getLogger(__name__)
@@ -48,84 +46,12 @@ def log_action(request, object, action_flag):
     )
 
 
-class MapEntityLayer(ModelViewMixin, GeoJSONLayerView):
-    """
-    Take a class attribute `model` with a `latest_updated` method used for caching.
-    """
-
-    force2d = True
-    srid = API_SRID
-
-    def __init__(self, *args, **kwargs):
-        super(MapEntityLayer, self).__init__(*args, **kwargs)
-        if self.model is None:
-            self.model = self.queryset.model
-        # Backward compatibility with django-geojson 1.X
-        # for JS ObjectsLayer and rando-trekking application
-        # TODO: remove when migrated
-        properties = dict([(k, k) for k in self.properties])
-        if 'id' not in self.properties:
-            properties['id'] = 'pk'
-        self.properties = properties
-
-    @classmethod
-    def get_entity_kind(cls):
-        return mapentity_models.ENTITY_LAYER
-
-    @view_permission_required()
-    @view_cache_latest()
-    def dispatch(self, *args, **kwargs):
-        return super(MapEntityLayer, self).dispatch(*args, **kwargs)
-
-    def render_to_response(self, context, **response_kwargs):
-        cache = get_cache(app_settings['GEOJSON_LAYERS_CACHE_BACKEND'])
-        key = '%s_%s_layer_json' % (self.request.LANGUAGE_CODE,
-                                    self.model._meta.module_name)
-
-        result = cache.get(key)
-        latest = self.model.latest_updated()
-
-        if result and latest:
-            cache_latest, content = result
-            # Not empty and still valid
-            if cache_latest and cache_latest >= latest:
-                return self.response_class(content=content, **response_kwargs)
-
-        response = super(MapEntityLayer, self).render_to_response(context, **response_kwargs)
-        cache.set(key, (latest, response.content))
-        return response
-
-
-class MapEntityList(ModelViewMixin, ListView):
+class MapEntityList(BaseListView, ListView):
     """
 
     A generic view list web page.
 
     """
-    model = None
-    filterform = None
-    columns = None
-
-    def __init__(self, *args, **kwargs):
-        super(MapEntityList, self).__init__(*args, **kwargs)
-
-        if self.model is None:
-            self.model = self.queryset.model
-
-        if self.filterform is None:
-            class filterklass(MapEntityFilterSet):
-                class Meta:
-                    model = self.model
-            self.filterform = filterklass
-        self._filterform = self.filterform(None, self.queryset)
-
-        if self.columns is None:
-            # All model fields except geometries
-            self.columns = [field.name for field in self.model._meta.fields
-                            if not isinstance(field, GeometryField)]
-            # Id column should be the first one
-            self.columns.remove('id')
-            self.columns.insert(0, 'id')
 
     def get_template_names(self):
         default = super(MapEntityList, self).get_template_names()
@@ -135,73 +61,45 @@ class MapEntityList(ModelViewMixin, ListView):
     def get_entity_kind(cls):
         return mapentity_models.ENTITY_LIST
 
-    def get_queryset(self):
-        queryset = super(MapEntityList, self).get_queryset()
-        # Filter queryset from possible serialized form
-        self._filterform = self.filterform(self.request.GET or None,
-                                           queryset=queryset)
-        return self._filterform.qs
-
     @view_permission_required(login_url='login')
     def dispatch(self, request, *args, **kwargs):
         # Save last list visited in session
-        request.session['last_list'] = request.path
+        # (only if viewing a true list, not an inherited ENTITY_JSON_LIST for ex.)
+        if self.__class__.get_entity_kind() == mapentity_models.ENTITY_LIST:
+            request.session['last_list'] = request.path
         return super(MapEntityList, self).dispatch(request, *args, **kwargs)
 
     def get_context_data(self, **kwargs):
         context = super(MapEntityList, self).get_context_data(**kwargs)
-        context['filterform'] = self._filterform
-        context['columns'] = self.columns
+        context['filterform'] = self._filterform  # From FilterListMixin
+        context['columns'] = self.columns  # From BaseListView
 
         context['create_label'] = self.get_model().get_create_label()
 
-        perm_create = self.model.get_permission_codename(mapentity_models.ENTITY_CREATE)
+        model = self.get_model()
+        perm_create = model.get_permission_codename(mapentity_models.ENTITY_CREATE)
         can_add = user_has_perm(self.request.user, perm_create)
         context['can_add'] = can_add
 
-        perm_export = self.model.get_permission_codename(mapentity_models.ENTITY_FORMAT_LIST)
+        perm_export = model.get_permission_codename(mapentity_models.ENTITY_FORMAT_LIST)
         can_export = user_has_perm(self.request.user, perm_export)
         context['can_export'] = can_export
 
         return context
 
 
-class MapEntityJsonList(JSONResponseMixin, MapEntityList):
-    """
-    Return objects list as a JSON that will populate the Jquery.dataTables.
+class MapEntityFormat(BaseListView, ListView):
     """
 
-    @classmethod
-    def get_entity_kind(cls):
-        return mapentity_models.ENTITY_JSON_LIST
+    Export the list to a particular format.
 
-    @view_permission_required()
-    def dispatch(self, *args, **kwargs):
-        return super(ListView, self).dispatch(*args, **kwargs)
-
-    def get_context_data(self, **kwargs):
-        """
-        Override the most important part of JSONListView... (paginator)
-        """
-        serializer = DatatablesSerializer()
-        return serializer.serialize(self.get_queryset(), fields=self.columns, model=self.model)
-
-
-class MapEntityFormat(MapEntityList):
-    """Make it  extends your EntityList"""
+    """
     DEFAULT_FORMAT = 'csv'
 
     @classmethod
     def get_entity_kind(cls):
         return mapentity_models.ENTITY_FORMAT_LIST
 
-    @view_permission_required()
-    def dispatch(self, *args, **kwargs):
-        return super(ListView, self).dispatch(*args, **kwargs)
-
-    def get_context_data(self, **kwargs):
-        return {}
-
     def render_to_response(self, context, **response_kwargs):
         """Delegate to the fmt view function found at dispatch time"""
         formats = {
@@ -219,31 +117,31 @@ class MapEntityFormat(MapEntityList):
             return HttpResponseBadRequest()
 
         filename = '%s-%s-list' % (datetime.now().strftime('%Y%m%d-%H%M'),
-                                   str(slugify(unicode(self.model._meta.verbose_name))))
+                                   str(slugify(unicode(self.get_model()._meta.verbose_name))))
         filename += '.%s' % extensions.get(fmt_str, fmt_str)
         response = formatter(request=self.request, context=context, **response_kwargs)
         response['Content-Disposition'] = 'attachment; filename=%s' % filename
         return response
 
     def csv_view(self, request, context, **kwargs):
-        serializer = CSVSerializer()
+        serializer = mapentity_serializers.CSVSerializer()
         response = HttpResponse(mimetype='text/csv')
         serializer.serialize(queryset=self.get_queryset(), stream=response,
-                             model=self.model, fields=self.columns, ensure_ascii=True)
+                             model=self.get_model(), fields=self.columns, ensure_ascii=True)
         return response
 
     def shape_view(self, request, context, **kwargs):
-        serializer = ZipShapeSerializer()
+        serializer = mapentity_serializers.ZipShapeSerializer()
         response = HttpResponse(mimetype='application/zip')
-        serializer.serialize(queryset=self.get_queryset(), model=self.model,
+        serializer.serialize(queryset=self.get_queryset(), model=self.get_model(),
                              stream=response, fields=self.columns)
         response['Content-length'] = str(len(response.content))
         return response
 
     def gpx_view(self, request, context, **kwargs):
-        serializer = GPXSerializer()
+        serializer = mapentity_serializers.GPXSerializer()
         response = HttpResponse(mimetype='application/gpx+xml')
-        serializer.serialize(self.get_queryset(), model=self.model, stream=response,
+        serializer.serialize(self.get_queryset(), model=self.get_model(), stream=response,
                              geom_field=app_settings['GEOM_FIELD_NAME'])
         return response
 
@@ -257,26 +155,26 @@ class MapEntityMapImage(ModelViewMixin, DetailView):
     def get_entity_kind(cls):
         return mapentity_models.ENTITY_MAPIMAGE
 
-    @view_permission_required()
-    def dispatch(self, *args, **kwargs):
-        return super(MapEntityMapImage, self).dispatch(*args, **kwargs)
-
     def render_to_response(self, context, **response_kwargs):
-        try:
-            obj = self.get_object()
-            obj.prepare_map_image(self.request.build_absolute_uri('/'))
+        obj = self.get_object()
+        if not obj.is_public():
+            if not self.request.user.is_authenticated():
+                raise PermissionDenied
+            if not self.request.user.has_perm('%s.read_%s' % (obj._meta.app_label, obj._meta.model_name)):
+                raise PermissionDenied
+        obj.prepare_map_image(self.request.build_absolute_uri('/'))
+        path = obj.get_map_image_path().replace(settings.MEDIA_ROOT, '').lstrip('/')
+        if settings.DEBUG or not app_settings['SENDFILE_HTTP_HEADER']:
+            response = static.serve(self.request, path, settings.MEDIA_ROOT)
+        else:
             response = HttpResponse(mimetype='image/png')
-            # Open image file, and writes to response
-            with open(obj.get_map_image_path(), 'rb') as f:
-                response.write(f.read())
-            return response
-        except Exception as e:
-            logger.exception(e)
-            return HttpResponseServerError(repr(e))
+            response[app_settings['SENDFILE_HTTP_HEADER']] = os.path.join(settings.MEDIA_URL_SECURE, path)
+        return response
 
 
 class MapEntityDocument(ModelViewMixin, DetailView):
     response_class = OdtTemplateResponse
+    with_html_attributes = True
 
     @classmethod
     def get_entity_kind(cls):
@@ -286,10 +184,12 @@ class MapEntityDocument(ModelViewMixin, DetailView):
         super(MapEntityDocument, self).__init__(*args, **kwargs)
         # Try to load template for each lang and object detail
         model = self.get_model()
-        name_for = lambda app, modelname, lang: "%s/%s%s%s.odt" % (app, modelname, lang, self.template_name_suffix)
         langs = ['_%s' % lang for lang, langname in app_settings['LANGUAGES']]
         langs.append('')   # Will also try without lang
 
+        def name_for(app, modelname, lang):
+            return "%s/%s%s%s.odt" % (app, modelname, lang, self.template_name_suffix)
+
         def smart_get_template():
             for appname, modelname in [(model._meta.app_label, model._meta.object_name.lower()),
                                        ("mapentity", "mapentity")]:
@@ -316,14 +216,15 @@ class MapEntityDocument(ModelViewMixin, DetailView):
 
         # Screenshot of object map is required, since present in document
         self.get_object().prepare_map_image(rooturl)
-        html = self.get_object().get_attributes_html(rooturl)
 
         context = super(MapEntityDocument, self).get_context_data(**kwargs)
         context['datetime'] = datetime.now()
         context['STATIC_URL'] = self.request.build_absolute_uri(settings.STATIC_URL)[:-1]
         context['MEDIA_URL'] = self.request.build_absolute_uri(settings.MEDIA_URL)[:-1]
         context['MEDIA_ROOT'] = settings.MEDIA_ROOT + '/'
-        context['attributeshtml'] = html
+        if self.with_html_attributes:
+            context['attributeshtml'] = self.get_object().get_attributes_html(self.request)
+        context['objecticon'] = os.path.join(settings.STATIC_ROOT, self.get_entity().icon_big)
         context['_'] = _
         return context
 
@@ -464,9 +365,7 @@ class MapEntityDetail(ModelViewMixin, DetailView):
         perm_update = self.get_model().get_permission_codename(mapentity_models.ENTITY_UPDATE)
         can_edit = user_has_perm(self.request.user, perm_update)
         context['can_edit'] = can_edit
-        context['can_read_attachment'] = user_has_perm(self.request.user, 'read_attachment')
-        context['can_add_attachment'] = user_has_perm(self.request.user, 'add_attachment')
-        context['can_delete_attachment'] = user_has_perm(self.request.user, 'delete_attachment')
+        context['attachment_form_class'] = AttachmentForm
 
         return context
 
@@ -534,17 +433,3 @@ class MapEntityDelete(ModelViewMixin, DeleteView):
 
     def get_success_url(self):
         return self.get_model().get_list_url()
-
-
-MAPENTITY_GENERIC_VIEWS = [
-    MapEntityLayer,
-    MapEntityList,
-    MapEntityJsonList,
-    MapEntityFormat,
-    MapEntityMapImage,
-    MapEntityDocument,
-    MapEntityCreate,
-    MapEntityDetail,
-    MapEntityUpdate,
-    MapEntityDelete,
-]
